@@ -7,6 +7,7 @@ A "builder" for Veracity's distributed build tracking feature.
 import os
 import sys
 import time
+import shutil
 import argparse
 import tempfile
 import subprocess
@@ -27,7 +28,7 @@ class Builder(_BaseRepositoryObject):
         @param repo: repository name
         @param config: common Config object
         """
-        super(Builder, self).__init__(repo=repo)
+        super(Builder, self).__init__(repo=repo, create=False)
         self.config = config
         self.count = 0
         self.stats = defaultdict(int)
@@ -63,22 +64,31 @@ class Builder(_BaseRepositoryObject):
                 # Check out build request's changeset
                 logging.info("found build request: {0}".format(request))
                 self.count += 1
-                with self.repo.checkout(tempfile.mkdtemp(), rev=request.cid) as work:
+                with self.repo.checkout(tempfile.mkdtemp(), rev=request.changeset.rev) as work:
+
                     # Update the configuration from the changeset
-                    path = common.find_config(work.path)
+                    work.chdir()
+                    path = common.find_config(work.path, walk=True)
                     if path:  # pragma: no cover - the test repo does not have a config
                         logging.info("updating the build configuration from the changeset...")
                         self.config.update(path)
-                    # Build state machine
+
+                    # Create a virtual environment
+                    logging.info("initial build state: [{0}]".format(status))
+                    work.chdir()
+                    self.create_virtualenv()
+
+                    # Build loop
                     while True:
+
                         # Enter the next build state
-                        request.update(self.config.enter(request.status))
-                        if request.status in self.config.ends:  # pragma: no cover - test build does not end normally
-                            break
+                        request.update(self.config.next(request.status))
+
                         # Run the command for the current state
                         path, command = self.config.get_build(request.status)
-                        work.chdir(path)
-                        success = self.call(command)
+                        work.chdir()
+                        success = self.call(path, command)
+
                         # Process the command results
                         if success:
                             request.update(self.config.exit(request.status))
@@ -86,6 +96,8 @@ class Builder(_BaseRepositoryObject):
                             request.update(self.config.fail(request.status))
                         if request.status in self.config.ends:
                             break
+
+                # Build completed
                 self.stats[request.status] += 1
                 logging.info("completed builds: {0}".format(self.count))
 
@@ -93,15 +105,22 @@ class Builder(_BaseRepositoryObject):
         """Perform one build iteration locally as a test.
         """
         for status in ([single] if single else self.config.starts):
+
+            # Create a virtual environment
+            logging.info("initial build state: [{0}]".format(status))
+            self.create_virtualenv()
+
+            # Build loop
             while True:
+
                 # Enter the next build state
-                status = self.config.enter(status)
-                if status in self.config.ends:  # pragma: no cover - the test repo does not have a config
-                    break
+                if not single:
+                    status = self.config.next(status)
+
                 # Run the command for the current state
                 path, command = self.config.get_build(status)
-                os.chdir(path)
-                success = self.call(command)
+                success = self.call(path, command)
+
                 # Process the command results
                 if success:
                     status = self.config.exit(status)
@@ -110,13 +129,38 @@ class Builder(_BaseRepositoryObject):
                 if single or status in self.config.ends:
                     break
 
-    @staticmethod
-    def call(command):
+            # Delete the virtual environment
+            self.delete_virtualenv()
+
+    def create_virtualenv(self):
+        """Create a virtual environment for the build.
+        """
+        if self.config.virtualenv:
+            logging.info("creating a virtual environment for build...")
+            command = "virtualenv {0}".format(self.config.virtualenv)
+            logging.info("$ {0}".format(command))
+            subprocess.call(command, shell=True)
+
+    def delete_virtualenv(self):
+        """Delete the created virtual environment.
+        """
+        if self.config.virtualenv:
+            logging.info("deleting the virtual environment...")
+            shutil.rmtree(self.config.virtualenv)
+
+    def call(self, path, command):
         """Run a build command in the current directory.
         @param command: shell string
         @return: indicates command returned no error
         """
         if command:
+            if path:
+                command = "cd {p} && {c}".format(p=path, c=command)
+            if self.config.virtualenv:  # pragma: no cover - OS determines coverage
+                if os.name == 'nt':
+                    command = "{v}\\Scripts\\activate && {c}".format(v=self.config.virtualenv, c=command)
+                else:
+                    command = ". {v}/bin/activate && {c}".format(v=self.config.virtualenv, c=command)
             logging.info("$ {0}".format(command))
             return subprocess.call(command, shell=True) == 0
         else:
@@ -144,7 +188,7 @@ def run(config, daemon=False, test=False):
                 logging.info("testing {b}...".format(b=builder))
                 builder.test()
             else:
-                logging.log(logging.DEBUG if daemon else logging.INFO, "running {b}...".format(b=builder))
+                logging.info("running {b}...".format(b=builder))
                 builder.run()
         if not daemon or test:
             break
