@@ -5,6 +5,7 @@ Classes representing all the "objects" used in Veracity.
 """
 
 import os
+import time
 import shutil
 import logging
 from collections import defaultdict, OrderedDict
@@ -12,7 +13,6 @@ from collections import defaultdict, OrderedDict
 from veracity import vv
 from veracity import vscript
 from veracity.vv import VeracityException
-from veracity.vscript import VeracityScriptException
 from veracity import settings
 
 
@@ -40,19 +40,15 @@ class _Base(object):  # pylint: disable=R0903
 class _BaseRepositoryObject(_Base):  # pylint: disable=R0903
     """Shared base class for objects that need a reference to the Repository."""
 
-    def __init__(self, **kwargs):
-        try:
-            repo = kwargs['repo']
-            if not repo:
-                raise KeyError
-        except KeyError:
-            logging.warning("'repo' argument missing")
-            self.repo = None
-        else:
+    def __init__(self, create=True, repo=None, **_kwargs):
+        if repo:
             if isinstance(repo, Repository):
                 self.repo = repo
             else:
-                self.repo = Repository(repo)
+                self.repo = Repository(repo, create=create)
+        else:
+            logging.warning("'repo' argument missing")
+            self.repo = None
 
     def __eq__(self, other):
         return self.repo == other.repo
@@ -84,19 +80,15 @@ class _BaseRepositorySpecObject(_BaseRepositoryObject):
 class _BaseWorkingCopyObject(_Base):  # pylint: disable=R0903
     """Shared base class for objects that need a reference to the WorkingCopy."""
 
-    def __init__(self, **kwargs):
-        try:
-            work = kwargs['work']
-            if not work:
-                raise KeyError
-        except KeyError:
-            logging.warning("'work' argument missing")
-            self.work = None
-        else:
+    def __init__(self, work=None, **_kwargs):
+        if work:
             if isinstance(work, WorkingCopy):
                 self.work = work
             else:
                 self.work = WorkingCopy(work)
+        else:
+            logging.warning("'work' argument missing")
+            self.work = None
 
     def __eq__(self, other):
         return self.work == other.work
@@ -111,8 +103,8 @@ def auto_pull(func):
     def wrapped(self, *args, **kwargs):
         """Wrapped method to call self.pull() before execution.
         """
-        if settings.AUTO_PULL:
-            self.pull()
+        if self._auto_pull:  # pylint: disable=W0212
+            self.pull(_auto=True)
         return func(self, *args, **kwargs)
 
     return wrapped
@@ -125,50 +117,46 @@ def auto_push(func):
         """Wrapped method to call self.push() after execution.
         """
         result = func(self, *args, **kwargs)
-        if settings.AUTO_PUSH:
-            self.push()
+        if self._auto_push:  # pylint: disable=W0212
+            self.push(_auto=True)
         return result
 
     return wrapped
 
 
-def requires_build(func):
-    """Decorator for methods that require an initialized build configuration.
-    """
-    def wrapped(self, *args, **kwargs):
-        """Wrapped method to show a better error message.
-        """
-        try:
-            return func(self, *args, **kwargs)
-        except VeracityScriptException as exception:
-            msg = str(exception).rsplit(':', 1)[-1].strip()
-            if "is null" in msg:
-                raise VeracityScriptException("build configuration not defined ({0})".format(msg))
-            else:
-                raise
-
-    return wrapped
-
-
 class Repository(_Base):
     """Represents a Veracity repository."""
 
-    def __init__(self, name, remote=None):
+    def __init__(self, name, remote=None, create=False, autosync=True):
         """Create/clone an existing repo or reference an existing.
         @param name: name of repository (exiting or new)
         @param remote: local or remote repository to clone
+        @param create: allow non-existing repositories to be created
         """
         self.name = name
         self.remote = remote
+        self._auto_pull = settings.AUTO_PULL and autosync
+        self._auto_push = settings.AUTO_PUSH and autosync
+        self._pull_time = 0
+        self._push_time = 0
         # Create a new repo if required
         logging.debug("checking for existing repo...")
         if self.name not in vv.parse(vv.run('repos')):
             if self.remote:
                 logging.info("cloning '{s}' to '{d}'...".format(s=self.remote, d=self.name))
                 vv.run('clone', self.remote, self.name)
-            else:
+            elif create:
                 logging.info("creating repo '{n}'...".format(n=self.name))
                 vv.run('repo', 'new', self.name)
+            else:
+                raise VeracityException("repository does not exist: {0}".format(name))
+
+    def __del__(self):
+        if settings.AUTO_PUSH:
+            try:
+                self.push(_auto=True)
+            except VeracityException:
+                pass
 
     def __repr__(self):
         return self._repr(self.name)
@@ -211,35 +199,51 @@ class Repository(_Base):
     def outgoing(self):  # pragma: no cover
         raise NotImplementedError
 
-    def pull(self, remote=None):
+    def pull(self, remote=None, _auto=False):
         """Pull from a remote repository.
         @param remote: repository name or URL
         """
+        if _auto and time.time() < self._pull_time + settings.MIN_DELAY:
+            logging.debug("skipped automatic pull")
+            return
         remote = remote or self.remote
         if remote:
-            logging.debug("pulling from repo '{r}'...".format(r=remote))
+            logging.log(logging.DEBUG if _auto else logging.INFO,
+                        "pulling from '{s}' to '{d}'...".format(s=remote, d=self))
             vv.run('pull', remote, dest=self.name)
+            self._pull_time = time.time()
         else:
             try:
-                logging.debug("pulling from default...")
+                logging.log(logging.DEBUG if _auto else logging.INFO,
+                            "pulling from default to '{d}'...".format(d=self))
                 vv.run('pull', dest=self.name)
             except VeracityException:
                 logging.warning("no remote repository for pull")
+            else:  # pragma: no cover - test repositories are explicitly cloned
+                self._pull_time = time.time()
 
-    def push(self, remote=None):
+    def push(self, remote=None, _auto=False):
         """Push to a remote repository.
         @param remote: repository name or URL
         """
+        if _auto and time.time() < self._push_time + settings.MIN_DELAY:
+            logging.debug("skipped automatic push")
+            return
         remote = remote or self.remote
         if remote:
-            logging.debug("pushing to repo '{r}'...".format(r=remote))
+            logging.log(logging.DEBUG if _auto else logging.INFO,
+                        "pushing from '{s}' to '{d}'...".format(s=self, d=remote))
             vv.run('push', remote, src=self.name)
+            self._push_time = time.time()
         else:
             try:
-                logging.debug("pushing to default...")
+                logging.log(logging.DEBUG if _auto else logging.INFO,
+                            "pushing from '{s}' to default...".format(s=self))
                 vv.run('push', src=self.name)
             except VeracityException:
                 logging.warning("no remote repository for push")
+            else:  # pragma: no cover - test repositories are explicitly cloned
+                self._push_time = time.time()
 
     @property
     @auto_pull
@@ -552,6 +556,7 @@ class Repository(_Base):
         @return: new WorkingCopy
         """
         rev, tag, branch = identify(default=True, **kwargs)
+        path = os.path.expanduser(path)
         logging.info("checking out '{i}' of '{r}' to '{p}'...".format(p=path, i=(rev or tag or branch), r=self))
         if not os.path.exists(path):
             os.makedirs(path)
@@ -568,7 +573,6 @@ class Repository(_Base):
         raise NotImplementedError
 
     @auto_push
-    @requires_build
     def request_build(self, name, series, environment, status, cid, branch=None):
         """Request a build of the repository.
         @param name: name for the build
@@ -587,7 +591,6 @@ class Repository(_Base):
                             status=status, branch=branch, repo=self)
 
     @auto_pull
-    @requires_build
     def get_build(self, environment, status, order=None):
         """Get the next pending build request in the repository.
         @param environment: alias for the current environment
@@ -896,7 +899,7 @@ class WorkingCopy(_BaseRepositoryObject):
             logging.debug("working copy is invalid")
             return False
 
-    def chdir(self, rel):
+    def chdir(self, rel='.'):
         """Change the current directory relative to the working copy.
         @param rel: path relative to the working copy
         """
@@ -1007,11 +1010,11 @@ class Changeset(_BaseRepositoryObject):
         return self._repr(dictionary, repo=self.repo)
 
     def __str__(self):
-        return self.rev
+        return self.rev[:10]
 
     def __eq__(self, other):
         if isinstance(other, basestring):
-            return str(self) == other
+            return self.rev == other
         elif not isinstance(other, Changeset):
             return False
         else:
@@ -1249,7 +1252,7 @@ class BuildRequest(_BaseRepositoryObject):
         """
         super(BuildRequest, self).__init__(**kwargs)
         self.bid = bid or '?' * 10
-        self.cid = cid or '?' * 10
+        self.changeset = Changeset(cid, repo=self.repo)
         # TODO: determine which attributes are actually needed by the class
         self.name = name
         self.series = series
@@ -1258,13 +1261,13 @@ class BuildRequest(_BaseRepositoryObject):
         self.branch = branch
 
     def __repr__(self):
-        return self._repr(self.bid, self.cid, name=self.name,
+        return self._repr(self.bid, self.changeset.rev, name=self.name,
                           series=self.series, environment=self.environment,
                           status=self.status, branch=self.branch,
                           repo=self.repo)
 
     def __str__(self):
-        return "'{b}' of '{c}'".format(b=self.bid[:10], c=self.cid[:10])
+        return "'{b}' of '{c}'".format(b=self.bid[:10], c=self.changeset)
 
     def __eq__(self, other):
         if isinstance(other, basestring):
