@@ -16,7 +16,7 @@ from veracity.vscript import VeracityScriptException
 from veracity.objects import identify, keywords
 from veracity.objects import Repository, WorkingCopy, Item, Changeset, Branch, Tag, Stamp, User
 from veracity.objects import BuildRequest
-from veracity.settings import VERBOSE_LOGGING_FORMAT, VERBOSE_LOGGING_LEVEL
+from veracity import settings
 
 from veracity.test import TEST_REPO_NAME, TEMP_REPO_NAME, TEST_REPO, REASON, log
 
@@ -36,23 +36,31 @@ class TestRepository(unittest.TestCase):
 
     def test_init_clone(self):
         """Verify a repository can be created as a clone."""
-        with Repository(TEMP_REPO_NAME, remote=TEST_REPO_NAME) as repo:
+        with Repository(TEMP_REPO_NAME, remote=TEST_REPO_NAME, autosync=False) as repo:
             log(repo)
             self.assertEqual(TEMP_REPO_NAME, str(repo))
-            repo.add_tag('_temptag', branch=Branch.MASTER)  # tag will get pushed to TEST_REPO
-        try:
-            self.assertIn('_temptag', TEST_REPO.tags)
-        except:
-            raise
-        finally:
-            TEST_REPO.remove_tag('_temptag')
+            with Repository(TEMP_REPO_NAME + '-clone', remote=repo) as repo2:
+                log(repo2)
+
+                repo2._push_time = 0  # force push, pylint: disable=W0212
+                repo2.add_tag('_temptag2', branch=Branch.MASTER)  # tag will get pushed to TEST_REPO
+                self.assertIn('_temptag2', repo.tags)
+
+                repo.add_tag('_temptag', branch=Branch.MASTER)
+                repo2._pull_time = 0  # force push, pylint: disable=W0212
+                self.assertIn('_temptag', repo2.tags)  # tag will get pulled  from TEST_REPO
+                self.assertIn('_temptag', repo.tags)
 
     def test_init_new(self):
         """Verify a new repository can be created."""
-        with Repository(TEMP_REPO_NAME) as repo:
+        with Repository(TEMP_REPO_NAME, create=True) as repo:
             log(repo)
             self.assertEqual(TEMP_REPO_NAME, str(repo))
 
+    def test_init_new_no_create(self):
+        """Verify an exception occurs when creation of new repositories is disabled."""
+        self.assertRaises(VeracityException, Repository, TEMP_REPO_NAME, create=False)
+
     def test_delete(self):
         """Verify a repository can not be used after it is deleted."""
         repo2 = Repository(TEMP_REPO_NAME, remote=TEST_REPO_NAME)
@@ -194,15 +202,25 @@ class TestChangeset(unittest.TestCase):
 class TestBranch(unittest.TestCase):
     """Tests on the Branch class."""  # pylint: disable=C0103
 
+    @classmethod
+    def setUpClass(cls):
+        if TEST_REPO:
+            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
+
+    @classmethod
+    def tearDownClass(cls):
+        if TEST_REPO:
+            cls.repo.delete()
+
     def test_repo_branches(self):
         """Verify the branches list can be retrieved from a repo."""
-        branches = TEST_REPO.branches
+        branches = self.repo.branches
         log(branches)
         self.assertIn('master', branches)
 
     def test_repo_add_move_branch(self):
         """Verify a branch can be added and moved."""
-        b = TEST_REPO.add_branch('_temp_branch', branch='master')
+        b = self.repo.add_branch('_temp_branch', branch='master')
         try:
             self.assertEqual('master', b.branch)
             b = b.move(rev=1)
@@ -215,11 +233,11 @@ class TestBranch(unittest.TestCase):
 
     def test_repo_close_open_branch(self):
         """Verify a branch can be closed and opened"""
-        b = Branch('master', repo=TEST_REPO, branch='master')
+        b = Branch('master', repo=self.repo, branch='master')
         b.close()
         try:
             b.close()  # already closed
-            self.assertNotIn(b, TEST_REPO.branches)
+            self.assertNotIn(b, self.repo.branches)
             b.reopen()
         except:
             logging.error("exception will be raised after cleanup...")
@@ -229,8 +247,8 @@ class TestBranch(unittest.TestCase):
 
     def test_eq(self):
         """Verify branches can be compared."""
-        b1 = Branch('a', repo=TEST_REPO, rev=1)
-        b2 = Branch('b', repo=TEST_REPO, rev=1)
+        b1 = Branch('a', repo=self.repo, rev=1)
+        b2 = Branch('b', repo=self.repo, rev=1)
         b3 = Branch('b', repo=None, rev=1)
         b4 = Branch('b', repo=None, rev=1)
         log(b1, b2, b3, b4)
@@ -241,7 +259,7 @@ class TestBranch(unittest.TestCase):
 
     def test_wc_branch(self):
         """Verify the branch can be set on a working copy."""
-        with TEST_REPO.checkout(tempfile.mkdtemp()) as wc:
+        with self.repo.checkout(tempfile.mkdtemp()) as wc:
             wc.branch = '_new_branch'
             self.assertEqual('_new_branch', wc.branch)
             branch = wc.attach('master')
@@ -249,7 +267,7 @@ class TestBranch(unittest.TestCase):
 
     def test_wc_branch_none(self):
         """Verify setting the working copy's branch to None detaches it."""
-        with TEST_REPO.checkout(tempfile.mkdtemp()) as wc:
+        with self.repo.checkout(tempfile.mkdtemp()) as wc:
             wc.branch = None
             self.assertEqual(None, wc.branch)
 
@@ -337,31 +355,41 @@ class TestItem(unittest.TestCase):
 class TestTag(unittest.TestCase):
     """Tests on the Tag class."""  # pylint: disable=C0103
 
+    @classmethod
+    def setUpClass(cls):
+        if TEST_REPO:
+            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
+
+    @classmethod
+    def tearDownClass(cls):
+        if TEST_REPO:
+            cls.repo.delete()
+
     def test_init(self):
         """Verify a tag can be created."""
-        tag = Tag('abc', repo=TEST_REPO, rev='123')
+        tag = Tag('abc', repo=self.repo, rev='123')
         self.assertEqual('abc', tag)
         self.assertNotEqual(None, tag)
 
     def test_repo_tags(self):
         """Verify a tag can be added and removed from a repo."""
         try:
-            TEST_REPO.add_tag('_temptag2', branch=Branch.MASTER)
-            tag = TEST_REPO.add_tag('_temptag2', branch=Branch.MASTER)  # returns already created tag
+            self.repo.add_tag('_temptag2', branch=Branch.MASTER)
+            tag = self.repo.add_tag('_temptag2', branch=Branch.MASTER)  # returns already created tag
             log(tag)
-            self.assertIn(tag, TEST_REPO.tags)
-            self.assertIn('_temptag2', TEST_REPO.tags)
+            self.assertIn(tag, self.repo.tags)
+            self.assertIn('_temptag2', self.repo.tags)
         except:
             logging.error("exception will be raised after cleanup...")
             raise
         finally:
-            TEST_REPO.remove_tag('_temptag2')
+            self.repo.remove_tag('_temptag2')
             tag.remove()  # already removed
 
     def test_repo_move_tag(self):
         """Verify a tag can be moved in a repository."""
         try:
-            tag = TEST_REPO.add_tag('_temptag', rev=2)
+            tag = self.repo.add_tag('_temptag', rev=2)
             log(tag)
             tag = tag.move(rev=1)
             self.assertEqual(1, tag.rev)
@@ -373,27 +401,37 @@ class TestTag(unittest.TestCase):
 
     def test_no_changset(self):
         """Verify a revision, tag, or branch is required to add a tag."""
-        self.assertRaises(ValueError, TEST_REPO.add_tag, '_invalid')
+        self.assertRaises(ValueError, self.repo.add_tag, '_invalid')
 
 
 @unittest.skipUnless(TEST_REPO, REASON)  # pylint: disable=R0904
 class TestStamp(unittest.TestCase):
     """Tests on the Stamp class."""  # pylint: disable=C0103
 
+    @classmethod
+    def setUpClass(cls):
+        if TEST_REPO:
+            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
+
+    @classmethod
+    def tearDownClass(cls):
+        if TEST_REPO:
+            cls.repo.delete()
+
     def test_init(self):
         """Verify a stamp can be created."""
-        stamp = Stamp('abc', repo=TEST_REPO, rev='123')
+        stamp = Stamp('abc', repo=self.repo, rev='123')
         self.assertEqual('abc', stamp)
         self.assertNotEqual(None, stamp)
 
     def test_repo_stamps(self):
         """Verify a stamp can be added and removed from a repo."""
         try:
-            TEST_REPO.add_stamp('_tempstamp', branch=Branch.MASTER)
-            stamp = TEST_REPO.add_stamp('_tempstamp', branch=Branch.MASTER)  # returns already created stamp
+            self.repo.add_stamp('_tempstamp', branch=Branch.MASTER)
+            stamp = self.repo.add_stamp('_tempstamp', branch=Branch.MASTER)  # returns already created stamp
             log(stamp)
-            self.assertIn(stamp, TEST_REPO.stamps)
-            self.assertIn('_tempstamp', TEST_REPO.stamps)
+            self.assertIn(stamp, self.repo.stamps)
+            self.assertIn('_tempstamp', self.repo.stamps)
         except:
             logging.error("exception will be raised after cleanup...")
             raise
@@ -402,7 +440,7 @@ class TestStamp(unittest.TestCase):
 
     def test_no_changset(self):
         """Verify a revision, tag, or branch is required to add a stamp."""
-        self.assertRaises(ValueError, TEST_REPO.add_stamp, '_invalid')
+        self.assertRaises(ValueError, self.repo.add_stamp, '_invalid')
 
 
 @unittest.skipUnless(TEST_REPO, REASON)  # pylint: disable=R0904
@@ -412,42 +450,48 @@ class TestUser(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         if TEST_REPO:
-            TEST_REPO.add_user('_tempuser')
+            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
+            cls.repo.add_user('_tempuser')
+
+    @classmethod
+    def tearDownClass(cls):
+        if TEST_REPO:
+            cls.repo.delete()
 
     def test_init(self):
         """Verify a user can be created."""
-        user = User('_tempuser', repo=TEST_REPO)
+        user = User('_tempuser', repo=self.repo)
         log(user)
         self.assertEqual('_tempuser', user)
-        self.assertNotEqual(User('_tempuser2', repo=TEST_REPO), user)
+        self.assertNotEqual(User('_tempuser2', repo=self.repo), user)
         self.assertNotEqual(None, user)
 
     def test_repo_users(self):
         """Verify the users list can be retrieved from a repository."""
-        log(TEST_REPO.users)
-        self.assertIn('_tempuser', TEST_REPO.users)
+        log(self.repo.users)
+        self.assertIn('_tempuser', self.repo.users)
 
     def test_repo_add_user_nobody(self):
         """Verify the 'nobody' user cannot be created."""
-        self.assertRaises(VeracityException, TEST_REPO.add_user, User.NOBODY)
+        self.assertRaises(VeracityException, self.repo.add_user, User.NOBODY)
 
     def test_repo_add_user_existing(self):
         """Verify attempting to add an existing user is handled."""
-        user = TEST_REPO.add_user('testuser')  # default user
+        user = self.repo.add_user('testuser')  # default user
         self.assertEqual('testuser', user)
 
     def test_repo_set_user(self):
         """Verify the current user can be set in a repository."""
-        TEST_REPO.user = '_tempuser'
-        self.assertEqual('_tempuser', TEST_REPO.user)
+        self.repo.user = '_tempuser'
+        self.assertEqual('_tempuser', self.repo.user)
 
     def test_repo_rename_user(self):
         """Verify a user can be renamed."""
-        u = User('testuser', repo=TEST_REPO)
+        u = User('testuser', repo=self.repo)
         u.set()
         u = u.rename('testuser2')
         try:
-            self.assertEqual('testuser2', TEST_REPO.user)
+            self.assertEqual('testuser2', self.repo.user)
         except:
             logging.error("exception will be raised after cleanup...")
             raise
@@ -456,7 +500,7 @@ class TestUser(unittest.TestCase):
 
     def test_repo_activate_user(self):
         """Verify a user can be deactivated and activated."""
-        u = User('testuser', repo=TEST_REPO)
+        u = User('testuser', repo=self.repo)
         u.deactivate()
         u.deactivate()  # redundant
         u.activate()
@@ -467,9 +511,15 @@ class TestUser(unittest.TestCase):
 class TestBuildRequest(unittest.TestCase):
     """Tests on the BuildRequest class."""  # pylint: disable=C0103
 
+    @classmethod
+    def setUpClass(cls):
+        if TEST_REPO:
+            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
+
     @classmethod
     def tearDownClass(cls):
-        del TEST_REPO.builds
+        if TEST_REPO:
+            cls.repo.delete()
 
     def test_init(self):
         """Verify a build request can be created."""
@@ -481,40 +531,44 @@ class TestBuildRequest(unittest.TestCase):
 
     def test_repo_request_build(self):
         """Verify a build can be requested in a repository."""
-        del TEST_REPO.builds
-        cs = TEST_REPO.get_heads(branch='master')[0]
-        request1 = TEST_REPO.request_build('testbuild', 'C', 'W', 'Q', cs.rev, branch='master')
+        del self.repo.builds
+        cs = self.repo.get_heads(branch='master')[0]
+        request1 = self.repo.request_build('testbuild', 'C', 'W', 'Q', cs.rev, branch='master')
         log(request1)
-        self.assertEqual(BuildRequest(request1.bid, None, repo=TEST_REPO), request1)
-        request2 = TEST_REPO.get_build('W', 'Q')
+        self.assertEqual(BuildRequest(request1.bid, None, repo=self.repo), request1)
+        request2 = self.repo.get_build('W', 'Q')
         log(request2)
         self.assertEqual(request1, request2)
 
     def test_repo_request_builds(self):
         """Verify multiple builds can be requested in a repository."""
-        del TEST_REPO.builds
-        cs = TEST_REPO.get_heads(branch='master')[0]
-        TEST_REPO.request_build('testbuild', 'C', 'W', 'Q', cs.rev)
-        TEST_REPO.request_build('testbuild2', 'N', 'L', 'Q', cs.rev, branch='master')
-        self.assertEqual(2, len(list(TEST_REPO.builds)))
+        del self.repo.builds
+        cs = self.repo.get_heads(branch='master')[0]
+        self.repo.request_build('testbuild', 'C', 'W', 'Q', cs.rev)
+        self.repo.request_build('testbuild2', 'N', 'L', 'Q', cs.rev, branch='master')
+        self.assertEqual(2, len(list(self.repo.builds)))
 
     def test_repo_request_build_invalid(self):
         """Verify an exception is raised when requesting an invalid build."""
-        self.assertRaises(VeracityScriptException, TEST_REPO.request_build, '_', 'C', 'W', 'Q', 'fake')
+        self.assertRaises(VeracityScriptException, self.repo.request_build, '_', 'C', 'W', 'Q', 'fake')
 
-    def test_repo_get_build_invalid(self):
-        """Verify an exception is raised when attempting to get an invalid environment."""
-        self.assertRaises(VeracityScriptException, TEST_REPO.get_build, 'INVALID', 'Q')
+    def test_repo_get_build_no_config(self):
+        """Verify None is return when getting builds of an invalid environment."""
+        self.assertRaises(VeracityScriptException, self.repo.get_build, 'W', 'INVALID')
+
+    def test_repo_get_build_no_env(self):
+        """Verify None is return when getting builds of an invalid environment."""
+        self.assertIsNone(self.repo.get_build('INVALID', 'Q'))
 
     def test_repo_get_build_none(self):
         """Verify nothing is returned when no builds are queued."""
-        del TEST_REPO.builds
-        self.assertEqual(None, TEST_REPO.get_build('M', 'Q'))
+        del self.repo.builds
+        self.assertEqual(None, self.repo.get_build('M', 'Q'))
 
     def test_update(self):
         """Verify a build request can be updated."""
-        cs = TEST_REPO.get_heads(branch='master')[0]
-        request = TEST_REPO.request_build('testbuild3', 'M', 'M', 'Q', cs.rev)
+        cs = self.repo.get_heads(branch='master')[0]
+        request = self.repo.request_build('testbuild3', 'M', 'M', 'Q', cs.rev)
         log(request)
         self.assertEqual('Q', request.status)
         request.update('B')
@@ -523,15 +577,14 @@ class TestBuildRequest(unittest.TestCase):
 
     def test_delete(self):
         """Verify a build request can be deleted."""
-        cs = TEST_REPO.get_heads(branch='master')[0]
-        request = TEST_REPO.request_build('testbuild4', 'C', 'L', 'Q', cs.rev)
+        cs = self.repo.get_heads(branch='master')[0]
+        request = self.repo.request_build('testbuild4', 'C', 'L', 'Q', cs.rev)
         log(request)
         request.delete()
         self.assertEqual(None, request.status)
         self.assertRaises(VeracityScriptException, request.update, 'B')
 
 
-@unittest.skipUnless(TEST_REPO, REASON)  # pylint: disable=R0904
 class TestFunctions(unittest.TestCase):
     """Tests on the module functions."""  # pylint: disable=C0103
 
@@ -557,5 +610,5 @@ class TestFunctions(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    logging.basicConfig(level=VERBOSE_LOGGING_LEVEL, format=VERBOSE_LOGGING_FORMAT)
+    logging.basicConfig(level=settings.VERBOSE_LOGGING_LEVEL, format=settings.VERBOSE_LOGGING_FORMAT)
     unittest.main()
